//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_Commands.h"
#include "Moa_DynamicArrays.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_commands {
//---------------------------------------------------------------------------
__fastcall TDocumentCommand::TDocumentCommand() :
	TCommand(),
	FDescription("No description")
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDocumentCommand::~TDocumentCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TItemsCommand::Initialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TItemsCommand::Uninitialize()
{
	if(FOwnItems) {
		for(int LI = 0; LI < FItems.Length; ++LI) {
			delete FItems[LI];
		}
	}
	FItems.Length = 0;
}
//---------------------------------------------------------------------------
void __fastcall TItemsCommand::AddItems()
{
	DocumentListModel->AddItems(Items);
}
//---------------------------------------------------------------------------
void __fastcall TItemsCommand::RemoveItems()
{
	DocumentListModel->DeleteItems(TDynamicArrayUtilities<int>::Sort(DocumentListModel->FindItems(Items)));
}
//---------------------------------------------------------------------------
__fastcall TItemsCommand::TItemsCommand(TDocumentListModel* ADocumentListModel, const DynamicArray<TDiskImage*>& AItems, bool AOwnItems) :
	TDocumentCommand(),
	FDocumentListModel(ADocumentListModel),
	FItems(AItems),
	FOwnItems(AOwnItems)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TItemsCommand::~TItemsCommand()
{
	Uninitialize();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TAddCommand::TAddCommand(TDocumentListModel* ADocumentListModel, const DynamicArray<TDiskImage*>& AItems) :
	TItemsCommand(ADocumentListModel, AItems, true)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TAddCommand::~TAddCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TAddCommand::Execute()
{
	Results = TCommandResults() << crClearQueue;
	AddItems();
	Results = TCommandResults() << crCanUndo;
}
//---------------------------------------------------------------------------
void __fastcall TAddCommand::Undo()
{
	Results = TCommandResults() << crClearUndo;
	RemoveItems();
	Results = TCommandResults() << crCanRedo;
}
//---------------------------------------------------------------------------
void __fastcall TAddCommand::Redo()
{
	Results = TCommandResults() << crClearRedo;
	AddItems();
	Results = TCommandResults() << crCanUndo;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TDeleteCommand::TDeleteCommand(TDocumentListModel* ADocumentListModel, const TIntegerDynArray& AIndexes) :
	TItemsCommand(ADocumentListModel, ADocumentListModel->GetItemsAt(AIndexes), false)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDeleteCommand::~TDeleteCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TDeleteCommand::Execute()
{
	Results = TCommandResults() << crClearQueue;
	RemoveItems();
	Results = TCommandResults() << crCanUndo;
}
//---------------------------------------------------------------------------
void __fastcall TDeleteCommand::Undo()
{
	Results = TCommandResults() << crClearUndo;
	AddItems();
	Results = TCommandResults() << crCanRedo;
}
//---------------------------------------------------------------------------
void __fastcall TDeleteCommand::Redo()
{
	Results = TCommandResults() << crClearRedo;
	RemoveItems();
	Results = TCommandResults() << crCanUndo;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TSetPropertySubCommand::TSetPropertySubCommand(TDocumentListModel* ADocumentListModel, TDiskImage* AItem) :
	TCommand(),
	FDocumentListModel(ADocumentListModel),
	FItem(AItem)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSetPropertySubCommand::~TSetPropertySubCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TSetPropertySubCommand::Execute()
{
	Results = TCommandResults() << crClearQueue;
	ExecuteEdit();
	DocumentListModel->ItemChange(DocumentListModel->Find(Item));
	Results = TCommandResults() << crCanUndo;
}
//---------------------------------------------------------------------------
void __fastcall TSetPropertySubCommand::Undo()
{
	Results = TCommandResults() << crClearUndo;
	UndoEdit();
	DocumentListModel->ItemChange(DocumentListModel->Find(Item));
	Results = TCommandResults() << crCanRedo;
}
//---------------------------------------------------------------------------
void __fastcall TSetPropertySubCommand::Redo()
{
	Results = TCommandResults() << crClearRedo;
	RedoEdit();
	DocumentListModel->ItemChange(DocumentListModel->Find(Item));
	Results = TCommandResults() << crCanUndo;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TSetNameSubCommand::ExecuteEdit()
{
	Item->Name = NewValue;
}
//---------------------------------------------------------------------------
void __fastcall TSetNameSubCommand::UndoEdit()
{
	Item->Name = OldValue;
}
//---------------------------------------------------------------------------
void __fastcall TSetNameSubCommand::RedoEdit()
{
	Item->Name = NewValue;
}
//---------------------------------------------------------------------------
__fastcall TSetNameSubCommand::TSetNameSubCommand(TDocumentListModel* ADocumentListModel, TDiskImage* AItem, const AnsiString& ANewValue) :
	TSetPropertySubCommand(ADocumentListModel, AItem),
	FOldValue(AItem->Name),
	FNewValue(ANewValue)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSetNameSubCommand::~TSetNameSubCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TSetWriteProtectedSubCommand::ExecuteEdit()
{
	Item->WriteProtected = NewValue;
}
//---------------------------------------------------------------------------
void __fastcall TSetWriteProtectedSubCommand::UndoEdit()
{
	Item->WriteProtected = OldValue;
}
//---------------------------------------------------------------------------
void __fastcall TSetWriteProtectedSubCommand::RedoEdit()
{
	Item->WriteProtected = NewValue;
}
//---------------------------------------------------------------------------
__fastcall TSetWriteProtectedSubCommand::TSetWriteProtectedSubCommand(TDocumentListModel* ADocumentListModel, TDiskImage* AItem, int ANewValue) :
	TSetPropertySubCommand(ADocumentListModel, AItem),
	FOldValue(AItem->WriteProtected),
	FNewValue(ANewValue)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSetWriteProtectedSubCommand::~TSetWriteProtectedSubCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TSetPropertyCommand::Initialize()
{
	try {
		FInvoker = new TInvoker();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TSetPropertyCommand::Uninitialize()
{
	delete FInvoker;
	FInvoker = NULL;
}
//---------------------------------------------------------------------------
__fastcall TSetPropertyCommand::TSetPropertyCommand(TDocumentListModel* ADocumentListModel) :
	TDocumentCommand(),
	FDocumentListModel(ADocumentListModel),
	FInvoker(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TSetPropertyCommand::~TSetPropertyCommand()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TSetPropertyCommand::Execute()
{
	Results = TCommandResults() << crCanUndo;
	while(Invoker->CanExecute()) {
		Invoker->Execute();
	}
}
//---------------------------------------------------------------------------
void __fastcall TSetPropertyCommand::Undo()
{
	Results = TCommandResults() << crCanRedo;
	while(Invoker->CanUndo()) {
		Invoker->Undo();
	}
}
//---------------------------------------------------------------------------
void __fastcall TSetPropertyCommand::Redo()
{
	Results = TCommandResults() << crCanUndo;
	while(Invoker->CanRedo()) {
		Invoker->Redo();
	}
}
//---------------------------------------------------------------------------
} // End namespace App_commands
//---------------------------------------------------------------------------

