//---------------------------------------------------------------------------
#ifndef App_CommandsH
#define App_CommandsH
//---------------------------------------------------------------------------
#include "App_Documents.h"
#include "Moa_CommandPattern.h"
//---------------------------------------------------------------------------
namespace App_commands {
//---------------------------------------------------------------------------
class TDocumentCommand : public TCommand
{
private:
	AnsiString FDescription;
protected:
public:
	__fastcall TDocumentCommand();
	virtual __fastcall ~TDocumentCommand();
	__property AnsiString Description = {read=FDescription, write=FDescription};
};
//---------------------------------------------------------------------------
class TItemsCommand : public TDocumentCommand
{
private:
	TDocumentListModel* FDocumentListModel;
	DynamicArray<TDiskImage*> FItems;
	bool FOwnItems;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
protected:
	virtual void __fastcall AddItems();
	virtual void __fastcall RemoveItems();
	__property TDocumentListModel* DocumentListModel = {read=FDocumentListModel};
	__property DynamicArray<TDiskImage*> Items = {read=FItems};
	__property bool OwnItems = {read=FOwnItems, write=FOwnItems};
public:
	__fastcall TItemsCommand(TDocumentListModel* ADocumentListModel, const DynamicArray<TDiskImage*>& AItems, bool AOwnItems);
	virtual __fastcall ~TItemsCommand();
};
//---------------------------------------------------------------------------
class TAddCommand : public TItemsCommand
{
private:
protected:
public:
	__fastcall TAddCommand(TDocumentListModel* ADocumentListModel, const DynamicArray<TDiskImage*>& AItems);
	virtual __fastcall ~TAddCommand();
	virtual void __fastcall Execute();
	virtual void __fastcall Undo();
	virtual void __fastcall Redo();
};
//---------------------------------------------------------------------------
class TDeleteCommand : public TItemsCommand
{
private:
protected:
public:
	__fastcall TDeleteCommand(TDocumentListModel* ADocumentListModel, const TIntegerDynArray& AIndexes);
	virtual __fastcall ~TDeleteCommand();
	virtual void __fastcall Execute();
	virtual void __fastcall Undo();
	virtual void __fastcall Redo();
};
//---------------------------------------------------------------------------
class TSetPropertySubCommand : public TCommand
{
private:
	TDocumentListModel* FDocumentListModel;
	TDiskImage* FItem;
protected:
	virtual void __fastcall ExecuteEdit() = 0;
	virtual void __fastcall UndoEdit() = 0;
	virtual void __fastcall RedoEdit() = 0;
	__property TDocumentListModel* DocumentListModel = {read=FDocumentListModel};
	__property TDiskImage* Item = {read=FItem};
public:
	__fastcall TSetPropertySubCommand(TDocumentListModel* ADocumentListModel, TDiskImage* AItem);
	virtual __fastcall ~TSetPropertySubCommand();
	virtual void __fastcall Execute();
	virtual void __fastcall Undo();
	virtual void __fastcall Redo();
};
//---------------------------------------------------------------------------
class TSetNameSubCommand : public TSetPropertySubCommand
{
private:
	AnsiString FOldValue;
	AnsiString FNewValue;
	__property AnsiString OldValue = {read=FOldValue};
	__property AnsiString NewValue = {read=FNewValue};
protected:
	virtual void __fastcall ExecuteEdit();
	virtual void __fastcall UndoEdit();
	virtual void __fastcall RedoEdit();
public:
	__fastcall TSetNameSubCommand(TDocumentListModel* ADocumentListModel, TDiskImage* AItem, const AnsiString& ANewValue);
	virtual __fastcall ~TSetNameSubCommand();
};
//---------------------------------------------------------------------------
class TSetWriteProtectedSubCommand : public TSetPropertySubCommand
{
private:
	int FOldValue;
	int FNewValue;
	__property int OldValue = {read=FOldValue};
	__property int NewValue = {read=FNewValue};
protected:
	virtual void __fastcall ExecuteEdit();
	virtual void __fastcall UndoEdit();
	virtual void __fastcall RedoEdit();
public:
	__fastcall TSetWriteProtectedSubCommand(TDocumentListModel* ADocumentListModel, TDiskImage* AItem, int ANewValue);
	virtual __fastcall ~TSetWriteProtectedSubCommand();
};
//---------------------------------------------------------------------------
class TSetPropertyCommand : public TDocumentCommand
{
private:
	TDocumentListModel* FDocumentListModel;
	TInvoker* FInvoker;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	__property TDocumentListModel* DocumentListModel = {read=FDocumentListModel};
protected:
public:
	__fastcall TSetPropertyCommand(TDocumentListModel* ADocumentListModel);
	virtual __fastcall ~TSetPropertyCommand();
	virtual void __fastcall Execute();
	virtual void __fastcall Undo();
	virtual void __fastcall Redo();
	__property TInvoker* Invoker = {read=FInvoker};
};
//---------------------------------------------------------------------------
} // End namespace App_commands
using namespace App_commands;
//---------------------------------------------------------------------------
#endif

